#!/usr/bin/env python
#
# Copyright (c) 2024, Marvell.
#
# SPDX-License-Identifier: MIT
#
# These scripts can be used for parsing and verifying LiquidSecurity HSM
# adapter attestation data; you may redistribute it and/or modify it under
# the terms of the MIT license.
#
"""Script for parsing a version 2 attestation file."""
import argparse
import binascii
import gzip
import struct

# Attestation structure is big endian.
# The response header is composed of the following:
# 1. Response code.
# 2. Request flags. These are set to include both the headers and the attributes
#    in the attestation.
# 3. Total size of the attestation.
# 4. Buffer size. The buffer contains the attestation attributes.
RESPONSE_HEADER = '>IIII'
# The info header is composed of the following:
# 1. Object version.
# 2. Request flags. These are set to include both the headers and the attributes
#    in the attestation.
# 3. Offset to the attributes of the first key.
# 4. Offset to the attributes of the second key.
INFO_HEADER = '>HHHH'
# The object header is composed of the handle, attribute count, and object size.
OBJ_HEADER = '>III'
# The TLV is composed of the object type and its size.
TLV = '>II'
# The attestation is signed with a 256-byte RSA signature.
SIGNATURE_SIZE = 256


def get_contents_decompress_if_needed(attestation_file):
  try:
    with gzip.open(attestation_file, 'rb') as f:
      return f.read()
  except IOError:
    with open(attestation_file, 'rb') as f:
      return f.read()


def parse_headers(attestation):
  _, _, totalsize, bufsize = struct.unpack_from(RESPONSE_HEADER, attestation, 0)
  attribute_offset = totalsize - (bufsize + SIGNATURE_SIZE)
  attest_data = attestation[attribute_offset:]
  _, _, offset1, offset2 = struct.unpack_from(INFO_HEADER, attest_data, 0)
  return attest_data, offset1, offset2


def parse(attest_data):
  _, attr_count, _ = struct.unpack_from(OBJ_HEADER, attest_data, 0)
  obj_header_size = struct.calcsize(OBJ_HEADER)
  attest_data = attest_data[obj_header_size:]
  attributes = {}
  while attr_count > 0:
    # Parse each Attribute.
    attr_type, attr_len = struct.unpack_from(TLV, attest_data, 0)
    attest_data = attest_data[struct.calcsize(TLV):]
    attributes['0x%04x' % attr_type] = binascii.hexlify(attest_data[:attr_len])
    attr_count -= 1
    attest_data = attest_data[attr_len:]
  return attributes


def print_attributes(attributes):
  for attr in sorted(attributes):
    print('%s: %s' % (attr, attributes[attr]))


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument('attestation_file', help=' The name of attestation file.')
  args = parser.parse_args()

  attestation = get_contents_decompress_if_needed(args.attestation_file)
  attest_data, offset1, offset2 = parse_headers(attestation)

  if offset2 > 0:
    print('Public Key Attestation')
    print_attributes(parse(attest_data[offset1:]))
    print('Private Key Attestation')
    print_attributes(parse(attest_data[offset2:]))
  else:
    print('Symmetric Key Attestation')
    print_attributes(parse(attest_data[offset1:]))


if __name__ == '__main__':
  main()

